@@ -4,6 +4,7 @@ import pandas as pd
 from .feature_engineer import FeatureEngineer
 from .feature_selector import FeatureSelector
 from .genetic_algorithm import GeneticAlgorithm
+from .swarm_intelligence import SwarmIntelligence
 from easyexplore.utils import Log
 from typing import Dict, List, Union
 
@@ -20,6 +21,7 @@ class FeatureLearning:
     Class for reinforcement feature engineering using supervised machine learning
     """
     def __init__(self,
+                 temp_dir: str = None,
                  feature_engineer: FeatureEngineer = None,
                  df: Union[dd.DataFrame, pd.DataFrame] = None,
                  file_path: str = None,
@@ -40,6 +42,9 @@ class FeatureLearning:
                  **kwargs
                  ):
         """
+        :param temp_dir: str
+            Path for the temporary feature files directory
+
         :param feature_engineer: FeatureEngineer
             Pre-defined FeatureEngineer object
 
@@ -85,6 +90,7 @@ class FeatureLearning:
         :param kwargs: dict
             Key-word arguments
         """
+        self.temp_dir: str = temp_dir
         self.imp_features: List[str] = []
         self.max_features: int = -1
         self.user_defined_max_features: int = max_features
@@ -106,7 +112,8 @@ class FeatureLearning:
                 self.output_path = '{}/'.format(self.output_path)
         self.save_temp_data: bool = save_temp_data
         if feature_engineer is None:
-            self.feature_engineer = FeatureEngineer(df=df,
+            self.feature_engineer = FeatureEngineer(temp_dir=temp_dir,
+                                                    df=df,
                                                     target_feature=target,
                                                     generate_new_feature=True,
                                                     id_features=kwargs.get('id_features'),
@@ -128,7 +135,8 @@ class FeatureLearning:
                                                     )
         else:
             self.feature_engineer = feature_engineer
-        self.feature_engineer.impute(multiple=True, multiple_meth='random', m=25, convergence_threshold=0.99)
+        if engineer_continuous:
+            self.feature_engineer.impute(multiple=True, multiple_meth='random', m=25, convergence_threshold=0.99)
         self.feature_engineer.reset_multi_threading()
         if self.engineer_time_disparity:
             self.feature_engineer.disparity(years=True if kwargs.get('years') is None else kwargs.get('years'),
@@ -147,7 +155,7 @@ class FeatureLearning:
         self.continuous_learning = None
         self.categorical_learning = None
 
-    def _evolve_feature_learning_ai(self, feature_type: str):
+    def _evolve_feature_learning_ai(self, feature_type: str, evolutionary_algorithm: str):
         """
         Evolve ai for feature learning using genetic algorithm
 
@@ -155,40 +163,77 @@ class FeatureLearning:
             Name of the feature type to engineer
                 -> continuous: (semi-) continuous features
                 -> categorical: categorical (nominal) features
+
+        :param evolutionary_algorithm: str
+            Name of the reinforced evolutionary algorithm
+                -> ga: Genetic Algorithm
+                -> si: Swarm Intelligence
         """
         if feature_type == 'continuous':
             Log(write=False, level='info').log(msg='Evolve feature learning ai for engineering (semi-) continuous features ...')
         else:
             Log(write=False, level='info').log(msg='Evolve feature learning ai for engineering categorical (one-hot encoded) features ...')
-        _feature_learner: GeneticAlgorithm = GeneticAlgorithm(mode='model',
-                                                              target=self.feature_engineer.get_target(),
-                                                              features=self.feature_engineer.get_predictors(),
-                                                              re_split_data=False if self.kwargs.get('re_split_data') is None else self.kwargs.get('re_split_data'),
-                                                              re_sample_cases=False if self.kwargs.get('re_sample_cases') is None else self.kwargs.get('re_sample_cases'),
-                                                              re_sample_features=True,
-                                                              max_features=self.max_features,
-                                                              labels=self.kwargs.get('labels'),
-                                                              models=['cat'] if self.kwargs.get('models') is None else self.kwargs.get('models'),
-                                                              model_params=None,
-                                                              burn_in_generations=-1 if self.kwargs.get('burn_in_generations') is None else self.kwargs.get('burn_in_generations'),
-                                                              warm_start=True if self.kwargs.get('warm_start') is None else self.kwargs.get('warm_start'),
-                                                              max_generations=2 if self.kwargs.get('max_generations_ai') is None else self.kwargs.get('max_generations_ai'),
-                                                              pop_size=64 if self.kwargs.get('pop_size') is None else self.kwargs.get('pop_size'),
-                                                              mutation_rate=0.1 if self.kwargs.get('mutation_rate') is None else self.kwargs.get('mutation_rate'),
-                                                              mutation_prob=0.8 if self.kwargs.get('mutation_prob') is None else self.kwargs.get('mutation_prob'),
-                                                              parents_ratio=0.5 if self.kwargs.get('parents_ratio') is None else self.kwargs.get('parents_ratio'),
-                                                              early_stopping=0 if self.kwargs.get('early_stopping') is None else self.kwargs.get('early_stopping'),
-                                                              convergence=False if self.kwargs.get('convergence') is None else self.kwargs.get('convergence'),
-                                                              timer_in_seconds=43200 if self.kwargs.get('timer_in_secondes') is None else self.kwargs.get('timer_in_secondes'),
-                                                              force_target_type=self.force_target_type,
-                                                              plot=False if self.kwargs.get('plot') is None else self.kwargs.get('plot'),
-                                                              output_file_path=self.kwargs.get('output_file_path'),
-                                                              multi_threading=False if self.kwargs.get('multi_threading') is None else self.kwargs.get('multi_threading'),
-                                                              multi_processing=False if self.kwargs.get('multi_processing') is None else self.kwargs.get('multi_processing'),
-                                                              log=False if self.kwargs.get('log') is None else self.kwargs.get('log'),
-                                                              verbose=0 if self.kwargs.get('verbose') is None else self.kwargs.get('verbose'),
-                                                              feature_engineer=self.feature_engineer
-                                                              )
+        if evolutionary_algorithm == 'ga':
+            _feature_learner: GeneticAlgorithm = GeneticAlgorithm(mode='model',
+                                                                  target=self.feature_engineer.get_target(),
+                                                                  features=self.feature_engineer.get_predictors(),
+                                                                  re_split_data=False if self.kwargs.get('re_split_data') is None else self.kwargs.get('re_split_data'),
+                                                                  re_sample_cases=False if self.kwargs.get('re_sample_cases') is None else self.kwargs.get('re_sample_cases'),
+                                                                  re_sample_features=True,
+                                                                  max_features=self.max_features,
+                                                                  labels=self.kwargs.get('labels'),
+                                                                  models=['cat'] if self.kwargs.get('models') is None else self.kwargs.get('models'),
+                                                                  model_params=None,
+                                                                  burn_in_generations=-1 if self.kwargs.get('burn_in_generations') is None else self.kwargs.get('burn_in_generations'),
+                                                                  warm_start=True if self.kwargs.get('warm_start') is None else self.kwargs.get('warm_start'),
+                                                                  max_generations=2 if self.kwargs.get('max_generations_ai') is None else self.kwargs.get('max_generations_ai'),
+                                                                  pop_size=64 if self.kwargs.get('pop_size') is None else self.kwargs.get('pop_size'),
+                                                                  mutation_rate=0.1 if self.kwargs.get('mutation_rate') is None else self.kwargs.get('mutation_rate'),
+                                                                  mutation_prob=0.85 if self.kwargs.get('mutation_prob') is None else self.kwargs.get('mutation_prob'),
+                                                                  parents_ratio=0.5 if self.kwargs.get('parents_ratio') is None else self.kwargs.get('parents_ratio'),
+                                                                  early_stopping=0 if self.kwargs.get('early_stopping') is None else self.kwargs.get('early_stopping'),
+                                                                  convergence=False if self.kwargs.get('convergence') is None else self.kwargs.get('convergence'),
+                                                                  timer_in_seconds=43200 if self.kwargs.get('timer_in_secondes') is None else self.kwargs.get('timer_in_secondes'),
+                                                                  force_target_type=self.force_target_type,
+                                                                  plot=False if self.kwargs.get('plot') is None else self.kwargs.get('plot'),
+                                                                  output_file_path=self.kwargs.get('output_file_path'),
+                                                                  multi_threading=False if self.kwargs.get('multi_threading') is None else self.kwargs.get('multi_threading'),
+                                                                  multi_processing=False if self.kwargs.get('multi_processing') is None else self.kwargs.get('multi_processing'),
+                                                                  log=False if self.kwargs.get('log') is None else self.kwargs.get('log'),
+                                                                  verbose=0 if self.kwargs.get('verbose') is None else self.kwargs.get('verbose'),
+                                                                  feature_engineer=self.feature_engineer
+                                                                  )
+        elif evolutionary_algorithm == 'si':
+            _feature_learner: SwarmIntelligence = SwarmIntelligence(mode='model',
+                                                                    target=self.feature_engineer.get_target(),
+                                                                    features=self.feature_engineer.get_predictors(),
+                                                                    re_split_data=False if self.kwargs.get('re_split_data') is None else self.kwargs.get('re_split_data'),
+                                                                    re_sample_cases=False if self.kwargs.get('re_sample_cases') is None else self.kwargs.get('re_sample_cases'),
+                                                                    re_sample_features=True,
+                                                                    max_features=self.max_features,
+                                                                    labels=self.kwargs.get('labels'),
+                                                                    models=['cat'] if self.kwargs.get('models') is None else self.kwargs.get('models'),
+                                                                    model_params=None,
+                                                                    burn_in_adjustments=-1 if self.kwargs.get('burn_in_adjustments') is None else self.kwargs.get('burn_in_adjustments'),
+                                                                    warm_start=True if self.kwargs.get('warm_start') is None else self.kwargs.get('warm_start'),
+                                                                    max_adjustments=2 if self.kwargs.get('max_adjustments_ai') is None else self.kwargs.get('max_adjustments_ai'),
+                                                                    pop_size=64 if self.kwargs.get('pop_size') is None else self.kwargs.get('pop_size'),
+                                                                    adjustment_rate=0.1 if self.kwargs.get('adjustment_rate') is None else self.kwargs.get('adjustment_rate'),
+                                                                    adjustment_prob=0.85 if self.kwargs.get('adjustment_prob') is None else self.kwargs.get('adjustment_prob'),
+                                                                    early_stopping=0 if self.kwargs.get('early_stopping') is None else self.kwargs.get('early_stopping'),
+                                                                    convergence=False if self.kwargs.get('convergence') is None else self.kwargs.get('convergence'),
+                                                                    timer_in_seconds=43200 if self.kwargs.get('timer_in_secondes') is None else self.kwargs.get('timer_in_secondes'),
+                                                                    force_target_type=self.force_target_type,
+                                                                    plot=False if self.kwargs.get('plot') is None else self.kwargs.get('plot'),
+                                                                    output_file_path=self.kwargs.get('output_file_path'),
+                                                                    multi_threading=False if self.kwargs.get('multi_threading') is None else self.kwargs.get('multi_threading'),
+                                                                    multi_processing=False if self.kwargs.get('multi_processing') is None else self.kwargs.get('multi_processing'),
+                                                                    log=False if self.kwargs.get('log') is None else self.kwargs.get('log'),
+                                                                    verbose=0 if self.kwargs.get('verbose') is None else self.kwargs.get('verbose'),
+                                                                    feature_engineer=self.feature_engineer
+                                                                    )
+        else:
+            raise FeatureLearningException('Reinfoced evolutionary algorithm ({}) not supported'.format(evolutionary_algorithm))
         _feature_learner.optimize()
         if feature_type == 'categorical':
             self.categorical_learning = _feature_learner.evolution
@@ -196,7 +241,7 @@ class FeatureLearning:
             self.continuous_learning = _feature_learner.evolution
         Log(write=False, level='error').log(msg='Feature learning ai evolved -> {}'.format(_feature_learner.evolution.get('model_name')))
 
-    def _feature_learning(self, feature_type: str):
+    def _feature_learning(self, feature_type: str, evolutionary_algorithm: str):
         """
         Run reinforcement feature learning based on feature types (categorical or continuous)
 
@@ -204,51 +249,97 @@ class FeatureLearning:
             Name of the feature type to engineer
                 -> continuous: (semi-) continuous features
                 -> categorical: categorical (nominal) features
+
+        :param evolutionary_algorithm: str
+            Name of the reinforced evolutionary algorithm
+                -> ga: Genetic Algorithm
+                -> si: Swarm Intelligence
         """
         Log(write=False, level='error').log(msg='Start feature engineering using {} features'.format('continuous original' if feature_type == 'continuous' else 'categorical one-hot-encoded'))
         if self.kwargs.get('mutation_prob') is None:
             self.kwargs.update(dict(mutation_prob=0.5))
+        if self.kwargs.get('adjustment_prob') is None:
+            self.kwargs.update(dict(adjustment_prob=0.5))
         if self.kwargs.get('max_generations') is None:
             self.kwargs.update(dict(max_generations=5))
+        if self.kwargs.get('max_adjustments') is None:
+            self.kwargs.update(dict(max_adjustments=5))
         if self.kwargs.get('parents_ratio') is None:
             self.kwargs.update(dict(parents_ratio=0.5))
-        _feature_learning_evolution: GeneticAlgorithm = GeneticAlgorithm(mode='feature_engineer',
-                                                                         feature_engineer=self.feature_engineer,
-                                                                         df=self.feature_engineer.get_data(),
-                                                                         target=self.feature_engineer.get_target(),
-                                                                         features=self.feature_engineer.get_predictors(),
-                                                                         re_split_data=False if self.kwargs.get('re_split_data') is None else self.kwargs.get('re_split_data'),
-                                                                         re_sample_cases=False if self.kwargs.get('re_sample_cases') is None else self.kwargs.get('re_sample_cases'),
-                                                                         re_sample_features=False,
-                                                                         max_features=self.max_features,
-                                                                         labels=self.kwargs.get('labels'),
-                                                                         models=[self.categorical_learning.get('model_name')] if feature_type == 'categorical' else [self.continuous_learning.get('model_name')],
-                                                                         model_params=self.categorical_learning.get('param') if feature_type == 'categorical' else self.continuous_learning.get('param'),
-                                                                         burn_in_generations=-1,
-                                                                         warm_start=False,
-                                                                         max_generations=self.kwargs.get('max_generations'),
-                                                                         pop_size=64 if self.kwargs.get('pop_size') is None else self.kwargs.get('pop_size'),
-                                                                         mutation_rate=0.1,
-                                                                         mutation_prob=self.kwargs.get('mutation_prob'),
-                                                                         parents_ratio=self.kwargs.get('parents_ratio'),
-                                                                         early_stopping=0,
-                                                                         convergence=False,
-                                                                         timer_in_seconds=43200 if self.kwargs.get('timer_in_secondes') is None else self.kwargs.get('timer_in_secondes'),
-                                                                         force_target_type=self.force_target_type,
-                                                                         plot=False if self.kwargs.get('plot') is None else self.kwargs.get('plot'),
-                                                                         output_file_path=self.kwargs.get('output_file_path'),
-                                                                         multi_threading=False if self.kwargs.get('multi_threading') is None else self.kwargs.get('multi_threading'),
-                                                                         multi_processing=False if self.kwargs.get('multi_processing') is None else self.kwargs.get('multi_processing'),
-                                                                         log=False if self.kwargs.get('log') is None else self.kwargs.get('log'),
-                                                                         verbose=0 if self.kwargs.get('verbose') is None else self.kwargs.get('verbose')
-                                                                         )
+        if evolutionary_algorithm == 'ga':
+            _feature_learning_evolution: GeneticAlgorithm = GeneticAlgorithm(mode='feature_engineer',
+                                                                             feature_engineer=self.feature_engineer,
+                                                                             df=self.feature_engineer.get_data(),
+                                                                             target=self.feature_engineer.get_target(),
+                                                                             features=self.feature_engineer.get_predictors(),
+                                                                             re_split_data=False if self.kwargs.get('re_split_data') is None else self.kwargs.get('re_split_data'),
+                                                                             re_sample_cases=False if self.kwargs.get('re_sample_cases') is None else self.kwargs.get('re_sample_cases'),
+                                                                             re_sample_features=False,
+                                                                             max_features=self.max_features,
+                                                                             labels=self.kwargs.get('labels'),
+                                                                             models=[self.categorical_learning.get('model_name')] if feature_type == 'categorical' else [self.continuous_learning.get('model_name')],
+                                                                             model_params=self.categorical_learning.get('param') if feature_type == 'categorical' else self.continuous_learning.get('param'),
+                                                                             burn_in_generations=-1,
+                                                                             warm_start=False,
+                                                                             max_generations=self.kwargs.get('max_generations'),
+                                                                             pop_size=64 if self.kwargs.get('pop_size') is None else self.kwargs.get('pop_size'),
+                                                                             mutation_rate=0.1,
+                                                                             mutation_prob=self.kwargs.get('mutation_prob'),
+                                                                             parents_ratio=self.kwargs.get('parents_ratio'),
+                                                                             early_stopping=0,
+                                                                             convergence=False,
+                                                                             timer_in_seconds=43200 if self.kwargs.get('timer_in_secondes') is None else self.kwargs.get('timer_in_secondes'),
+                                                                             force_target_type=self.force_target_type,
+                                                                             plot=False if self.kwargs.get('plot') is None else self.kwargs.get('plot'),
+                                                                             output_file_path=self.kwargs.get('output_file_path'),
+                                                                             multi_threading=False if self.kwargs.get('multi_threading') is None else self.kwargs.get('multi_threading'),
+                                                                             multi_processing=False if self.kwargs.get('multi_processing') is None else self.kwargs.get('multi_processing'),
+                                                                             log=False if self.kwargs.get('log') is None else self.kwargs.get('log'),
+                                                                             verbose=0 if self.kwargs.get('verbose') is None else self.kwargs.get('verbose')
+                                                                             )
+        elif evolutionary_algorithm == 'si':
+            _feature_learning_evolution: SwarmIntelligence = SwarmIntelligence(mode='feature_engineer',
+                                                                               feature_engineer=self.feature_engineer,
+                                                                               df=self.feature_engineer.get_data(),
+                                                                               target=self.feature_engineer.get_target(),
+                                                                               features=self.feature_engineer.get_predictors(),
+                                                                               re_split_data=False if self.kwargs.get('re_split_data') is None else self.kwargs.get('re_split_data'),
+                                                                               re_sample_cases=False if self.kwargs.get('re_sample_cases') is None else self.kwargs.get('re_sample_cases'),
+                                                                               re_sample_features=False,
+                                                                               max_features=self.max_features,
+                                                                               labels=self.kwargs.get('labels'),
+                                                                               models=[self.categorical_learning.get('model_name')] if feature_type == 'categorical' else [self.continuous_learning.get('model_name')],
+                                                                               model_params=self.categorical_learning.get('param') if feature_type == 'categorical' else self.continuous_learning.get('param'),
+                                                                               burn_in_adjustments=-1,
+                                                                               warm_start=False,
+                                                                               max_adjustments=self.kwargs.get('max_adjustments'),
+                                                                               pop_size=64 if self.kwargs.get('pop_size') is None else self.kwargs.get('pop_size'),
+                                                                               adjustment_rate=0.1,
+                                                                               adjustment_prob=self.kwargs.get('adjustment_prob'),
+                                                                               early_stopping=0,
+                                                                               convergence=False,
+                                                                               timer_in_seconds=43200 if self.kwargs.get('timer_in_secondes') is None else self.kwargs.get('timer_in_secondes'),
+                                                                               force_target_type=self.force_target_type,
+                                                                               plot=False if self.kwargs.get('plot') is None else self.kwargs.get('plot'),
+                                                                               output_file_path=self.kwargs.get('output_file_path'),
+                                                                               multi_threading=False if self.kwargs.get('multi_threading') is None else self.kwargs.get('multi_threading'),
+                                                                               multi_processing=False if self.kwargs.get('multi_processing') is None else self.kwargs.get('multi_processing'),
+                                                                               log=False if self.kwargs.get('log') is None else self.kwargs.get('log'),
+                                                                               verbose=0 if self.kwargs.get('verbose') is None else self.kwargs.get('verbose')
+                                                                               )
+        else:
+            raise FeatureLearningException('Reinforced evolutionary algorithm ({}) not supported'.format(evolutionary_algorithm))
         _feature_learning_evolution.optimize()
         self.evolved_features.extend(_feature_learning_evolution.evolved_features)
         self.feature_engineer = _feature_learning_evolution.feature_engineer
-        Log(write=False, level='error').log(msg='Generated {} engineered features'.format(len(_feature_learning_evolution.mutated_features.get('child'))))
+        if evolutionary_algorithm == 'ga':
+            _generated_features: List[str] = _feature_learning_evolution.mutated_features.get('child')
+        else:
+            _generated_features: List[str] = _feature_learning_evolution.adjusted_features.get('to')
+        Log(write=False, level='error').log(msg='Generated {} engineered features'.format(len(_generated_features)))
         if self.keep_fittest_only:
             Log(write=False, level='error').log(msg='Selected {} fittest features'.format(len(_feature_learning_evolution.evolved_features)))
-            _erase: Dict[str, List[str]] = dict(features=list(set(_feature_learning_evolution.mutated_features.get('child')).difference(_feature_learning_evolution.evolved_features)))
+            _erase: Dict[str, List[str]] = dict(features=list(set(_generated_features).difference(_feature_learning_evolution.evolved_features)))
             if len(_erase.get('features')) > 0:
                 self.feature_engineer.clean(markers=_erase)
         del _feature_learning_evolution
@@ -259,7 +350,7 @@ class FeatureLearning:
         """
         self.feature_engineer.label_encoder(encode=True)
         self.feature_engineer.date_categorizer()
-        self.feature_engineer.binning(supervised=True, optimal=True, optimal_meth='bayesian_blocks')
+        #self.feature_engineer.binning(supervised=True, optimal=True, optimal_meth='bayesian_blocks')
         self.feature_engineer.one_hot_encoder(threshold=self.kwargs.get('threshold'))
         _features: List[str] = self.feature_engineer.get_features(feature_type='ordinal') + self.feature_engineer.get_features(feature_type='categorical')
         self.feature_engineer.set_predictors(features=_features, exclude_original_data=True)
@@ -331,8 +422,8 @@ class FeatureLearning:
                 if self.train_continuous_critic:
                     self._feature_critic()
                 self._pre_define_max_features(feature_type='continuous', scale=True if self.user_defined_max_features <= 1 else False)
-                self._evolve_feature_learning_ai(feature_type='continuous')
-                self._feature_learning(feature_type='continuous')
+                self._evolve_feature_learning_ai(feature_type='continuous', evolutionary_algorithm='ga')
+                self._feature_learning(feature_type='continuous', evolutionary_algorithm='ga')
             else:
                 Log(write=False, env='dev').log(msg='Not enough continuous or ordinal features to efficiently run reinforcement feature learning framework')
         if self.engineer_categorical:
@@ -357,8 +448,8 @@ class FeatureLearning:
                         if self.train_categorical_critic:
                             self._feature_critic()
                         self._pre_define_max_features(feature_type='categorical', scale=True if self.user_defined_max_features <= 1 else False)
-                        self._evolve_feature_learning_ai(feature_type='categorical')
-                        self._feature_learning(feature_type='categorical')
+                        self._evolve_feature_learning_ai(feature_type='categorical', evolutionary_algorithm='ga')
+                        self._feature_learning(feature_type='categorical', evolutionary_algorithm='ga')
                         self.feature_engineer.merge_engineer(feature_engineer_file_path='{}feature_learning.p'.format(self.output_path))
                     else:
                         Log(write=False, env='dev').log(msg='Not enough categorical features to efficiently run reinforcement feature learning framework')
@@ -368,16 +459,69 @@ class FeatureLearning:
                         self._feature_critic()
                     self._pre_define_max_features(feature_type='categorical',
                                                   scale=True if self.user_defined_max_features <= 1 else False)
-                    self._evolve_feature_learning_ai(feature_type='categorical')
-                    self._feature_learning(feature_type='categorical')
+                    self._evolve_feature_learning_ai(feature_type='categorical', evolutionary_algorithm='ga')
+                    self._feature_learning(feature_type='categorical', evolutionary_algorithm='ga')
                 else:
                     Log(write=False, env='dev').log(msg='Not enough categorical features to efficiently run reinforcement feature learning framework')
         if len(self.evolved_features) > 0:
             self.feature_engineer.set_predictors(features=list(set(self.evolved_features)), exclude_original_data=False)
         return self.feature_engineer
 
-    def nas(self):
+    def si(self) -> FeatureEngineer:
         """
-        Run neural architecture search for continuous feature engineering
+        Run swarm intelligence for optimizing semi-continuous, continuous and (one-hot-encoded) categorical feature engineering
+
+        :return: FeatureEngineer
+            FeatureEngineer object containing the hole feature engineering, meta data included
         """
-        raise NotImplementedError('Neural architecture search (NAS) for feature learning not implemented')
+        if self.engineer_continuous:
+            if self.generate_continuous:
+                self._generate_continuous_features()
+            if len(self.feature_engineer.get_predictors()) >= 4:
+                if self.train_continuous_critic:
+                    self._feature_critic()
+                self._pre_define_max_features(feature_type='continuous', scale=True if self.user_defined_max_features <= 1 else False)
+                self._evolve_feature_learning_ai(feature_type='continuous', evolutionary_algorithm='si')
+                self._feature_learning(feature_type='continuous', evolutionary_algorithm='si')
+            else:
+                Log(write=False, env='dev').log(msg='Not enough continuous or ordinal features to efficiently run reinforcement feature learning framework')
+        if self.engineer_categorical:
+            if self.generate_categorical:
+                self._generate_categorical_features()
+            if self.save_temp_data:
+                if self.output_path is None:
+                    Log(write=False, level='info').log(msg='No output path found for writing temporary data for applying one-hot merging')
+                else:
+                    self.feature_engineer.save(file_path='{}feature_learning.p'.format(self.output_path),
+                                               cls_obj=True,
+                                               overwrite=True,
+                                               create_dir=False
+                                               )
+                    del self.feature_engineer
+                    self.feature_engineer: FeatureEngineer = FeatureEngineer(feature_engineer_file_path='{}feature_learning.p'.format(self.output_path))
+                    _continuous_features: List[str] = self.feature_engineer.get_feature_types().get('continuous')
+                    self.feature_engineer.clean(markers=dict(features=_continuous_features))
+                    _remaining_non_categorical_features: List[str] = self.feature_engineer.get_feature_types().get('date')# + self.feature_engineer.get_feature_types().get('text')
+                    self.feature_engineer.clean(markers=dict(features=_remaining_non_categorical_features))
+                    if len(self.feature_engineer.get_predictors()) >= 4:
+                        if self.train_categorical_critic:
+                            self._feature_critic()
+                        self._pre_define_max_features(feature_type='categorical', scale=True if self.user_defined_max_features <= 1 else False)
+                        self._evolve_feature_learning_ai(feature_type='categorical', evolutionary_algorithm='si')
+                        self._feature_learning(feature_type='categorical', evolutionary_algorithm='si')
+                        self.feature_engineer.merge_engineer(feature_engineer_file_path='{}feature_learning.p'.format(self.output_path))
+                    else:
+                        Log(write=False, env='dev').log(msg='Not enough categorical features to efficiently run reinforcement feature learning framework')
+            else:
+                if len(self.feature_engineer.get_predictors()) >= 4:
+                    if self.train_categorical_critic:
+                        self._feature_critic()
+                    self._pre_define_max_features(feature_type='categorical',
+                                                  scale=True if self.user_defined_max_features <= 1 else False)
+                    self._evolve_feature_learning_ai(feature_type='categorical', evolutionary_algorithm='si')
+                    self._feature_learning(feature_type='categorical', evolutionary_algorithm='si')
+                else:
+                    Log(write=False, env='dev').log(msg='Not enough categorical features to efficiently run reinforcement feature learning framework')
+        if len(self.evolved_features) > 0:
+            self.feature_engineer.set_predictors(features=list(set(self.evolved_features)), exclude_original_data=False)
+        return self.feature_engineer
